//
//  Note+Helper.swift
//  SlipboxProject
//
//  Created by Karin Prater on 19.11.22.
//

import Foundation
import CoreData

extension Note {
    
    var title: String {
        get { self.title_ ?? "" }
        set { self.title_ = newValue  }
    }
    
    var creationDate: Date {
        get { creationDate_ ?? Date() }
    }
    
    var status: Status {
        get {
            if let rawStatus = status_,
              let status = Status(rawValue: rawStatus){
                return status
            } else {
                return Status.draft
            }
        }
        set {
            status_ = newValue.rawValue
        }
    }
    
    var formattedBodyText: NSAttributedString {
        get {
            formattedBodyText_?.toAttributedString() ?? NSAttributedString(string: "")
        }
        set {
            formattedBodyText_ =  newValue.toData()
            bodyText_ = newValue.string.lowercased()
        }
    }
    
    var bodyText: String {
        get { bodyText_ ?? "" }
    }
    
    var backLinkedNotes: Set<Note> {
        get { (backLinkedNotes_ as? Set<Note>) ?? [] }
        set { backLinkedNotes_ = newValue as NSSet }
    }
    
    var linkedNotes: Set<Note> {
        get { (linkedNotes_ as? Set<Note>) ?? [] }
        set { linkedNotes_ = newValue as NSSet }
    }
    
    var keywords: Set<Keyword> {
        get { (keywords_ as? Set<Keyword>) ?? [] }
        set { keywords_ = newValue as NSSet }
    }
    
    //MARK: - init
    
    convenience init(title: String, context: NSManagedObjectContext) {
        self.init(context: context)
        self.title = title
    }
 
    public override func awakeFromInsert() {
        self.creationDate_ = Date()
        self.status = Status.draft
    }
    
   static func fetch(_ predicate: NSPredicate = .all) -> NSFetchRequest<Note> {
        let request = NSFetchRequest<Note>(entityName: "Note")
        request.sortDescriptors = [NSSortDescriptor(keyPath: \Note.creationDate_, ascending: true)]
        request.predicate = predicate
        
        return request
    }
    
    static func delete(note: Note) {
        guard let context = note.managedObjectContext else { return }
        
        context.delete(note)
    }
    
    //MARK: - preview
    
    static func example() -> Note {
        let context = PersistenceController.preview.container.viewContext
        
        let note = Note(title: "my note", context: context)
        note.formattedBodyText = NSAttributedString(string: defaultText)
        
        let keys = Keyword.exampleArray()
        for key in keys {
            note.keywords.insert(key)
        }
        
        return note
    }
    
    static let defaultText = "Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum."
    
}

extension Note: Comparable {
    public static func < (lhs: Note, rhs: Note) -> Bool {
        lhs.creationDate < rhs.creationDate
    }
}

//MARK: - define my string constants

struct NoteProperties {
    static let title = "title_"
    static let bodyText = "bodyText_"
    static let status = "status_"
    static let creationDate = "creationDate_"
    
    static let folder = "folder"
    static let keywords = "keywords_"
}
